package cz.cuni.dsrg.sofa.netbeans.tirbrowser;

import org.openide.actions.PropertiesAction;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Sheet;
import org.openide.util.HelpCtx;
import org.openide.util.actions.SystemAction;

import java.io.IOException;

/** A simple node with no children.
 *
 * @author Petr Hnetynka
 */


public class TIRContainedNode extends AbstractNode 
{
    String tirName;
    SOFA.SOFAnode.Made.TIR.Contained tirObject;



    public TIRContainedNode(SOFA.SOFAnode.Made.TIR.Contained c, String name) throws java.rmi.RemoteException
    {
        super (Children.LEAF);
        
        tirObject = c;
        setIconBase ("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/TIRContainedNodeIcon");
        // Whatever is most relevant to a user:
        setDefaultAction (SystemAction.get (PropertiesAction.class));
        // Set FeatureDescriptor stuff:
        
        if (name != null) 
        {
            tirName = name;
        } 
        else 
        {
            SOFA.SOFAnode.Made.TIR.Identification id =  tirObject.get_identification();
            tirName = MiscHelps.printKind(tirObject.get_def_kind());
            tirName = tirName + " \"" + id.name();
            String ver = id.version();
            if (ver!=null && ver.length()!=0)
            tirName = tirName +"?" +ver;
            tirName = tirName + "\"";
        }
        
        setName (tirName); // or, super.setName if needed
        //setDisplayName (NbBundle.getMessage (TIRContainedNode.class, "LBL_node"));
        //setShortDescription (NbBundle.getMessage (TIRContainedNode.class, "HINT_node"));
        // Add cookies, e.g.:
        /*
        getCookieSet ().add (new OpenCookie () 
        {
		public void open () 
		{
		    // Open something useful...
		}
	    });
        */
    }



    // Create the popup menu:
    protected SystemAction[] createActions () 
    {
        return new SystemAction[] 
        {
                   // SystemAction.get (MyFavoriteAction.class),
                   // null,                     // separator
                   /* according to what it can do:
                   SystemAction.get (CutAction.class),
                   SystemAction.get (CopyAction.class),
                   null,
                   SystemAction.get (DeleteAction.class),
                   SystemAction.get (RenameAction.class),
                   null,
                   */
                   //SystemAction.get (ToolsAction.class),
                   //null,
                   SystemAction.get (PropertiesAction.class),
          };
    }



    public HelpCtx getHelpCtx () 
    {
        return HelpCtx.DEFAULT_HELP;
        // When you have help, change to:
        // return new HelpCtx (TIRContainedNode.class);
    }

    
    
    // RECOMMENDED - handle cloning specially (so as not to invoke the overhead of FilterNode):
    /*
    public Node cloneNode () 
    {
	// Try to pass in similar constructor params to what you originally got:
  return new TIRContainedNode ();
    }
    */

    
    // Create a property sheet:
    protected Sheet createSheet () 
    {
	      Sheet sheet = super.createSheet ();
	      // Make sure there is a "Properties" set:
	      Sheet.Set props = sheet.get (Sheet.PROPERTIES); // get by name, not display name
	      if (props == null) 
	      {
	          props = Sheet.createPropertiesSet ();
	          sheet.put (props);
	      }
        try 
        {
            SOFA.SOFAnode.Made.TIR.DefinitionKind dk = tirObject.get_def_kind();
            int dkInt = dk.value();
            SOFA.SOFAnode.Made.TIR.Identification id = tirObject.get_identification();
            props.put (new StringReadOnlyProp("DefKind", "Definition kind", null,MiscHelps.printKind(dk)));
	          props.put (new StringReadOnlyProp("Name", null, null,id.name()));
            props.put (new StringReadOnlyProp("Version", null, null,id.version()));
            props.put (new StringArrayReadOnlyProp("Tags", null, null,id.tag()));
            if (dkInt == SOFA.SOFAnode.Made.TIR.DefinitionKind.dk_Operation)
            {
                SOFA.SOFAnode.Made.TIR.ParamDescr[] params = ((SOFA.SOFAnode.Made.TIR.OperationDef)tirObject).params();
                String[] types = new String [params.length];
                for (int i=0;i<params.length;i++) 
                {
                    types[i] = MiscHelps.cdlType2String(params[i].type());
                }
                props.put (new StringArrayReadOnlyProp("Parameters", null, null,types));
                props.put (new StringReadOnlyProp("Result", "Return type", null, MiscHelps.cdlType2String(((SOFA.SOFAnode.Made.TIR.OperationDef)tirObject).result())));
            }
            if (dkInt == SOFA.SOFAnode.Made.TIR.DefinitionKind.dk_Attribute)
            {
                props.put (new StringReadOnlyProp("Type", null, null, MiscHelps.cdlType2String(((SOFA.SOFAnode.Made.TIR.AttributeDef)tirObject).type())));
            }
            if (dkInt == SOFA.SOFAnode.Made.TIR.DefinitionKind.dk_Typedef)
            {
                props.put (new StringReadOnlyProp("Type", "Original type", null, MiscHelps.cdlType2String(((SOFA.SOFAnode.Made.TIR.TypedefDef)tirObject).original_type())));
            }
            if (dkInt == SOFA.SOFAnode.Made.TIR.DefinitionKind.dk_Inst)
            {
                props.put (new StringReadOnlyProp("Type", null, null, MiscHelps.cdlType2String(((SOFA.SOFAnode.Made.TIR.InstDef)tirObject).type())));
            }
            if (dkInt == SOFA.SOFAnode.Made.TIR.DefinitionKind.dk_Property)
            {
                props.put (new StringReadOnlyProp("Type", null, null, MiscHelps.cdlType2String(((SOFA.SOFAnode.Made.TIR.PropertyDef)tirObject).type())));
            }
        } 
        catch (java.rmi.RemoteException e2) 
        {
            MiscHelps.exception(e2);
        } 
        catch (SOFA.SOFAnode.Made.TIR.TIRExceptLock e3)
        {
            MiscHelps.exception(e3);
        }
        return sheet;
    }

    // Handle renaming:
    /*
    public boolean canRename () 
    {
	return true;
    }
    public void setName (String nue) 
    {
	// Update visible name, fire property changes:
	super.setName (nue);
	// perform additional actions, i.e. rename underlying object
    }
    */

    // Handle deleting:
    
    public boolean canDestroy () 
    {
	      return true;
    }
    
    public void destroy () throws IOException 
    {
	// Actually remove the node itself and fire property changes:
	      super.destroy ();
	// perform additional actions, i.e. delete underlying object
        tirObject = null;
    }
    

    // Handle copying and cutting specially:
    /*
    public boolean canCopy () 
    {
	return true;
    }
    
    public boolean canCut () 
    {
	return true;
    }
    
    public Transferable clipboardCopy () 
    {
	// Add to, do not replace, the default node copy flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCopy ());
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) 
	{
		protected Object getData () 
		{
                   return TIRContainedNode.this.getDisplayName ();
		}
	});
	return et;
    }
    
    
    public Transferable clipboardCut () 
    {
	// Add to, do not replace, the default node cut flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCut ());
	// This is not so useful because this node will not be destroyed afterwards
	// (it is up to the paste type to decide whether to remove the "original",
	// and it is not safe to assume that getData will only be called once):
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) 
	{
		protected Object getData () 
		{
                 return TIRContainedNode.this.getDisplayName ();
		}
	    });
	return et;
    }
    */

    
    
    // Permit user to customize whole node at once (instead of per-property):
    /*
    public boolean hasCustomizer () 
    {
	return true;
    }
    
    
    public Component getCustomizer () 
    {
	return new MyCustomizingPanel (this);
    }
    */

}
