package cz.cuni.dsrg.sofa.netbeans.tirbrowser;

import SOFA.SOFAnode.Made.TIR.DefinitionKind;
import org.openide.ErrorManager;
import org.openide.TopManager;
import org.openide.actions.PropertiesAction;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Sheet;
import org.openide.util.HelpCtx;
import org.openide.util.actions.SystemAction;

import java.rmi.RemoteException;



/** A node with some children.
 *
 * @author Petr Hnetynka
 */
public class TIRContainerNode extends AbstractNode
{
    protected SOFA.SOFAnode.Made.TIR.Container tirObject;
    private String tirName;

    public TIRContainerNode (SOFA.SOFAnode.Made.TIR.Container c, String nodeName) throws java.rmi.RemoteException
    {
        super (new TIRContainerChildren ());
        tirObject = c;
        //SOFA.SOFAnode.Made.TIR.Identification id = ((SOFA.SOFAnode.Made.TIR.Contained)tirObject).get_identification();
        //tirName = id.name()+?id.version();
        
        switch (tirObject.get_def_kind().value()) 
        {
            case DefinitionKind.dk_Architecture:
                setIconBase ("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/ArchitectureNodeIcon");
                break;
            case DefinitionKind.dk_Frame:
                setIconBase ("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/FrameNodeIcon");
                break;
            case DefinitionKind.dk_Interface:
                setIconBase ("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/InterfaceNodeIcon");
                break;
            case DefinitionKind.dk_Provider:
                setIconBase("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/ProviderNodeIcon");
                break;
            default:
                setIconBase ("cz/cuni/dsrg/sofa/netbeans/resources/tirbrowser/TIRContainerNodeIcon");
        }
        
        // Whatever is most relevant to a user:
        //setDefaultAction (Properties.class);
        // Set FeatureDescriptor stuff:
        
        if (nodeName != null) 
        {
            tirName = nodeName;
        } 
        else 
        {
            SOFA.SOFAnode.Made.TIR.Identification id = ((SOFA.SOFAnode.Made.TIR.Contained) tirObject).get_identification();
            tirName = MiscHelps.printKind(tirObject.get_def_kind());
            tirName = tirName + " \"" + id.name();
            String ver = id.version();
            if (ver!=null && ver.length()!=0)
            tirName = tirName +"?" +ver;
            tirName = tirName + "\"";
       }
        
        setName (tirName); // or, super.setName if needed
        //setDisplayName (NbBundle.getMessage (TIRContainerNode.class, "LBL_node"));
        //setShortDescription (NbBundle.getMessage (TIRContainerNode.class, "HINT_node"));
        // Add cookies, e.g.:
        /*
        getCookieSet ().add (new OpenCookie () 
        {
		public void open () 
		{
		    // Open something useful...
		}
	});


        */
        // Make reorderable:
        // getCookieSet ().add (new ReorderMe ());
        createChildren();
        //addNodeListener(new TIRContainerListener());
        getCookieSet().add(new TIRContainerRefresh(this));
    }

    
    // Create the popup menu:
    protected SystemAction[] createActions () 
    {

        try
        {
            if(tirObject.get_def_kind().value()!=DefinitionKind.dk_Architecture)
                return new SystemAction[]
                      {
                                 SystemAction.get (RefreshNodeAction.class),
                                 null,
                                 SystemAction.get (PropertiesAction.class),
                       };

          else
              return new SystemAction[]
                    {
                               SystemAction.get (GenerateAction.class),
                               SystemAction.get (RefreshNodeAction.class),
                               null,
                               SystemAction.get (PropertiesAction.class),
                     };

        }
        catch (RemoteException e)
        {
            MiscHelps.message("Exception "+ e.getMessage());
            e.printStackTrace();
            return null;
        }
    }



    public HelpCtx getHelpCtx () 
    {
        return HelpCtx.DEFAULT_HELP;
        // When you have help, change to:
        // return new HelpCtx (TIRContainerNode.class);
    }

    
    protected TIRContainerChildren getTIRContainerChildren () 
    {
        return (TIRContainerChildren) getChildren ();
    }

    
    // RECOMMENDED - handle cloning specially (so as not to invoke the overhead of FilterNode):
    /*
    public Node cloneNode () 
    {
	// Try to pass in similar constructor params to what you originally got:
        return new TIRContainerNode ();
    }
    */

    // Create a property sheet:
    protected Sheet createSheet () 
    {
	      Sheet sheet = super.createSheet ();
	      // Make sure there is a "Properties" set:
	      Sheet.Set props = sheet.get (Sheet.PROPERTIES); // get by name, not display name
	      if (props == null) 
	      {
	          props = Sheet.createPropertiesSet ();
	          sheet.put (props);
	      }
        try {
            SOFA.SOFAnode.Made.TIR.DefinitionKind dk = tirObject.get_def_kind();
            int dkInt = dk.value();
            SOFA.SOFAnode.Made.TIR.Identification id = ((SOFA.SOFAnode.Made.TIR.Contained)tirObject).get_identification();
            props.put (new StringReadOnlyProp("DefKind", "Definition kind", null,MiscHelps.printKind(dk)));
	          props.put (new StringReadOnlyProp("Name", null, null,id.name()));
            if (!MiscHelps.isModuleLike(dk)) 
            {
                props.put (new StringReadOnlyProp("Version", null, null,id.version()));
                props.put (new StringArrayReadOnlyProp("Tags", null, null,id.tag()));
            }

            if (dkInt == SOFA.SOFAnode.Made.TIR.DefinitionKind.dk_Architecture)
            {
                props.put (new StringReadOnlyProp("System", null, null,String.valueOf(((SOFA.SOFAnode.Made.TIR.ArchitectureDef)tirObject).is_system())));
            }

            if (dkInt == SOFA.SOFAnode.Made.TIR.DefinitionKind.dk_Frame)
            {
                props.put (new StringReadOnlyProp("System", null, null,String.valueOf(((SOFA.SOFAnode.Made.TIR.FrameDef)tirObject).is_system())));
            }
        
        /*} catch (NoSuchMethodException e1) 
        {
            MiscHelps.exception(e1);*/
        } catch (java.rmi.RemoteException e2) 
        {
            MiscHelps.exception(e2);
        }
        return sheet;
    }
    

   
    // Permit new subnodes to be created:
    /*
    public NewType[] getNewTypes () 
    {
	return new NewType[] { new NewType () 
	{
		public String getName () 
		{
                    return NbBundle.getMessage (TIRContainerNode.class, "LBL_NewType");
		}
		// If you have help:
		// public HelpCtx getHelpCtx () 
		{
                //     return TIRContainerNode.class.getName () + ".newType";
		// }
		public void create () throws IOException 
		{
		    // do whatever you need, e.g.:
                    getTIRContainerChildren ().addKey (someNewKey);
		    // Throw an IOException if you are creating an underlying
		    // object and this fails.
		}
	    }
	};
    }
    */

    
    
    // Permit things to be pasted into this node:
    /*
    protected void createPasteTypes (final Transferable t, List l) 
    {
	// Make sure to pick up super impl, which adds intelligent node paste type:
	super.createPasteTypes (t, l);
	if (t.isDataFlavorSupported (DataFlavor.stringFlavor)) 
	{
	    l.add (new PasteType () 
	    {
		    public String getName () 
		    {
                        return NbBundle.getMessage (TIRContainerNode.class, "LBL_PasteType");
		    }
		    // If you have help:
		    // public HelpCtx getHelpCtx () 
		    {
                    //     return TIRContainerNode.class.getName () + ".pasteType";
		    // }
		    public Transferable paste () throws IOException 
		    {
			try 
			{
			    String data = (String) t.getTransferData (DataFlavor.stringFlavor);
			    // Or, you can look for nodes and related things in the transferable, using e.g.:
			    // Node n = NodeTransfer.node (t, NodeTransfer.COPY);
			    // Node[] ns = NodeTransfer.nodes (t, NodeTransfer.MOVE);
			    // MyCookie cookie = (MyCookie) NodeTransfer.cookie (t, NodeTransfer.COPY, MyCookie.class);
			    // do something, e.g.:
                            getTIRContainerChildren ().addKey (data);
			    // Throw an IOException if you are creating an underlying
			    // object and this fails.
			    // To leave the clipboard as is:
			    return null;
			    // To clear the clipboard:
			    // return ExTransferable.EMPTY;
			}
			catch (UnsupportedFlavorException ufe) 
			{
			    // Should not happen, since t said it supported this flavor, but:
			    throw new IOException (ufe.getMessage ());
			}
		    }
		});
	}
    }
    */

    // Handle renaming:
    /*
    public boolean canRename () 
    {
	return true;
    }

    public void setName (String nue) 
    {
	// Update visible name, fire property changes:
	super.setName (nue);
	// perform additional actions, i.e. rename underlying object
    }
    */

    // Handle deleting:
    


    public boolean canDestroy () 
    {
        //MiscHelps.message("Test delete");
        return false;
    }


    public void destroy () throws java.io.IOException 
    {
	// Actually remove the node itself and fire property changes:
	      super.destroy ();
	// perform additional actions, i.e. delete underlying object
	// (and don't forget about objects represented by your children!)
        TIRContainerChildren ch = getTIRContainerChildren();
        ch.remove(ch.getNodes());
    }
    


    // Handle copying and cutting specially:
    /*
    public boolean canCopy () 
    {
	return true;
    }

    public boolean canCut () 
    {
	return true;
    }

    public Transferable clipboardCopy () 
    {
	// Add to, do not replace, the default node copy flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCopy ());
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) 
	{
		protected Object getData () 
		{
                    return TIRContainerNode.this.getDisplayName ();
		}
	    });
	return et;
    }
    
    
    public Transferable clipboardCut () 
    {
	// Add to, do not replace, the default node cut flavor:
	ExTransferable et = ExTransferable.create (super.clipboardCut ());
	// This is not so useful because this node will not be destroyed afterwards
	// (it is up to the paste type to decide whether to remove the "original",
	// and it is not safe to assume that getData will only be called once):
	et.put (new ExTransferable.Single (DataFlavor.stringFlavor) 
	{
		protected Object getData () 
		{
                    return TIRContainerNode.this.getDisplayName ();
		}
	    });
	return et;
    }
    */

    // Permit user to customize whole node at once (instead of per-property):
    /*
    public boolean hasCustomizer () 
    {
	return true;
    }

    public Component getCustomizer () 
    {
	return new MyCustomizingPanel (this);
    }
    */

    // Permit node to be reordered (you may also want to put
    // MoveUpAction and MoveDownAction on the subnodes, if you can,
    // but ReorderAction on the parent is enough):
    /*
    private class ReorderMe extends Index.Support 
    {
      
	public Node[] getNodes () 
	{
            return TIRContainerNode.this.getChildren ().getNodes ();
	}
      
	public int getNodesCount () 
	{
	    return getNodes ().length;
	}
      
	// This assumes that there is exactly one child node per key.
	// If you are using e.g. Children.Array, you can use shortcut implementations
	// of the Index cookie.
	public void reorder (int[] perm) 
	{
	    // Remember: {2, 0, 1} cycles three items forwards.
            List old = TIRContainerNode.this.getTIRContainerChildren ().myKeys;
	    if (list.size () != perm.length) throw new IllegalArgumentException ();
	    List nue = new ArrayList (perm.length);
	    for (int i = 0; i < perm.length; i++)
		nue.set (i, old.get (perm[i]));
            TIRContainerNode.this.getTIRContainerChildren ().setKeys (nue);
	}
      
    }
    */

    
    
    protected void createChildren()
    {
        /*
        TopManager tp = TopManager.getDefault();
        try 
        {
            TIRContainerChildren children = getTIRContainerChildren();
            SOFA.SOFAnode.Made.TIR.Contained[] cnt = tirObject.contents(null);
            for (int i=0;i<cnt.length;i++) 
            {
                SOFA.SOFAnode.Made.TIR.Identification id = cnt[i].get_identification();
                String name = MiscHelps.printKind(cnt[i].get_def_kind());
                
                name = name + " \"" + id.name();
                String ver = id.version();
                if (ver!=null && ver.length()!=0)
                    name = name +"?" +ver;
                name = name + "\"";
                
                if (cnt[i] instanceof SOFA.SOFAnode.Made.TIR.Container)
                    children.addChild (new TIRContainerNode((SOFA.SOFAnode.Made.TIR.Container) cnt[i], name));
                else
                    children.addChild (new TIRContainedNode(cnt[i], name));
            }
        } 
        catch (RemoteException e) 
        {
            tp.notify(new NotifyDescriptor.Exception(e));
        } 
        catch (SOFA.SOFAnode.Made.TIR.TIRExceptLock e)
        {
            tp.notify(new NotifyDescriptor.Exception(e));
        }

         */
        //java.util.Collection cc = new java.util.LinkedList();
        ChildrenCreateList cc = new ChildrenCreateList();
        TopManager tp = TopManager.getDefault();
        try 
        {
            TIRContainerChildren children = getTIRContainerChildren();
            SOFA.SOFAnode.Made.TIR.Contained[] cnt = tirObject.contents(null);
            for (int i=0;i<cnt.length;i++) 
            {
                
                SOFA.SOFAnode.Made.TIR.Identification id = cnt[i].get_identification();
                /*
                String name = MiscHelps.printKind(cnt[i].get_def_kind());
                
                name = name + " \"" + id.name();
                String ver = id.version();
                if (ver!=null && ver.length()!=0)
                    name = name +"?" +ver;
                name = name + "\"";
                 */
                String name = id.name();
                if (!cc.contains(name)) 
                {
                    //System.out.println("+++ "+name+" +++");
                    cc.add(new TIRContainerKey(name, tirObject.lookup_name(name)));
                }
            }
            children.setMyKeys(cc.list);
        }
         catch (RemoteException e) 
         {
            ErrorManager.getDefault().notify(e);
        } 
        catch (SOFA.SOFAnode.Made.TIR.TIRExceptLock e)
        {
            ErrorManager.getDefault().notify(e);
        }
    }
    
    
    static class ChildrenCreateList 
    {
        java.util.LinkedList list;
        java.util.LinkedList names;
        
        public ChildrenCreateList() 
        {
            list = new java.util.LinkedList();
            names = new java.util.LinkedList();
        }
        
        public void add(TIRContainerKey k) 
        {
            list.add(k);
            names.add(k.name);
        }
        
        public boolean contains(String k) 
        {
            return names.contains(k);
        }
    }
    
    
}
