package cz.cuni.dsrg.sofa.netbeans.tirbrowser;

import org.openide.nodes.Children;
import org.openide.nodes.Node;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

/** List of children of a containing node.
 * Remember to document what your permitted keys are!
 *
 * @author Petr Hnetynka
 */

public class TIRRootNodeChildren extends Children.Keys 
{
    /** Optional holder for the keys, to be used when changing them dynamically. */
    protected List myKeys;

    public TIRRootNodeChildren () 
    {
        myKeys = null;
    }

    protected void addNotify () 
    {
        if (myKeys != null) return;
        myKeys = new LinkedList ();
        // add whatever keys you need
        setKeys (myKeys);
    }

    protected void removeNotify () 
    {
        myKeys = null;
        setKeys (Collections.EMPTY_SET);
    }

    protected Node[] createNodes (Object key) 
    {
        // interpret your key here...usually one node generated, but could be zero or more
        return new Node[] {/* new MyNode ((MyParameter) key) */};
    }

    protected boolean addChild (Object newKey) 
    {
        addNotify ();
        String key =((Node)newKey).getName();
        if (myKeys.contains(key))
            return false;
	      myKeys.add (key);
        add(new Node[] { (Node)newKey} ); 
	      refreshKey (newKey);
        return true;
    }
    
    /** if this is unique key */
    protected boolean testChildKey(String newKey) 
    {
        addNotify ();
        return !myKeys.contains(newKey);
    }
    
    public void removeKey(Object key) 
    {
        if (myKeys != null) 
        {
            if (myKeys.remove(key)) 
            {
                if (myKeys.size()==0) 
                {
                    myKeys = null;
                    setKeys (Collections.EMPTY_SET);
                } else 
                {
                    setKeys(myKeys);
                }
            }
        }
    }
    

    /** Optional accessor method for keys, for use by the container node or maybe subclasses. */
    /*
    protected void setKeys (Collection keys) 
    {
	myKeys = new LinkedList ();
	myKeys.addAll (keys);
	super.setKeys (keys);
    }
    */

    // Could also write e.g. removeKey to be used by the nodes in this children.
    // Or, could listen to changes in their status (NodeAdapter.nodeDestroyed)
    // and automatically remove them from the keys list here. Etc.

}
