package cz.cuni.dsrg.sofa.netbeans.visualdesigner.designer;

import javax.swing.*;
import java.awt.event.ActionEvent;

/**
 * This is a helper class that creates the toolbar for the component
 * designer.
 *
 * @author Michael Cifka
 */
public class Actions
{

    public Actions(ArchitectureEditor editor)
    {
        this.editor = editor;
    }

    private ArchitectureEditor editor;

    private Action selectionAction = new AbstractAction(
            "Selection Mode", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/selection.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            editor.getEventLayer().setMode(EventLayer.SELECTION_MODE);
        }
    };

    private Action instanceAction = new AbstractAction(
            "Selection Mode", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/instance.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            editor.getEventLayer().setMode(EventLayer.INSTANCE_MODE);
        }
    };

    private Action provisionAction = new AbstractAction(
            "Add Provision", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/provision.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            editor.setInterfaceMode(ArchitectureEditor.INTERFACE_PROVISION);
            editor.getEventLayer().setMode(EventLayer.INTERFACE_MODE);
        }
    };

    private Action requirementAction = new AbstractAction(
            "Add Requirement", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/requirement.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            editor.setInterfaceMode(ArchitectureEditor.INTERFACE_REQUIREMENT);
            editor.getEventLayer().setMode(EventLayer.INTERFACE_MODE);
        }
    };

    private Action deleteAction = new AbstractAction(
            "Selection Mode", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/delete.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            editor.delete();
        }
    };

    private Action connectAction = new AbstractAction(
            "Selection Mode", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/connect.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            editor.getEventLayer().setMode(EventLayer.BINDING_MODE);
        }
    };
    private Action disconnectAction = new AbstractAction(
            "Selection Mode", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/disconnect.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            editor.unbind();
        }
    };

    public JToolBar createToolBar()
    {
        JToolBar toolBar = new JToolBar(JToolBar.HORIZONTAL);


        toolBar.setToolTipText("Toolbar of actions of the component visual designer");

        toolBar.add(selectionAction).setToolTipText("Select an element");
        toolBar.addSeparator();
        toolBar.add(instanceAction).setToolTipText("Add a new subcomponent");
        toolBar.add(provisionAction).setToolTipText("Add a new provided interface to a subcomponent");
        toolBar.add(requirementAction).setToolTipText("Add a new required interface to a subcomponent");
        toolBar.addSeparator();
        toolBar.add(deleteAction).setToolTipText("Delete the selected element");
        toolBar.addSeparator();
        toolBar.add(connectAction).setToolTipText("Create a tie between a provided and a required interface");
        toolBar.add(disconnectAction).setToolTipText("Remove a tie between interfaces");
        toolBar.setFloatable(false);


        return toolBar;
    }
}
