package cz.cuni.dsrg.sofa.netbeans.visualdesigner.designer;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;

import javax.swing.*;
import java.awt.*;

/**
 * A visually designable component representing the frame of the parent component.
 *
 * @author Michael Cifka
 */
public class GrFrame extends JPanel implements VisualDesign
{

    private static final int INSETS = Constants.FRAME_INSETS;
    private static final int WIDTH = Constants.FRAME_WIDTH;
    private static final int HEIGHT = Constants.FRAME_HEIGHT;

    public GrFrame(Component component, ArchitectureEditor editor)
    {
        this.editor = editor;
        this.component = component;

        Dimension size = component.getLayout().getStandaloneSize();
        super.setBounds(INSETS, INSETS, size.width, size.height);
        editor.resetSize(size.width + 2 * INSETS, size.height + 2 * INSETS);

        setOpaque(true);
        setBackground(Constants.FRAME_COLORS[component.getColorIndex()]);
        setBorder(BorderFactory.createLineBorder(Color.black));
        setLayout(new BorderLayout());

        title = new JLabel();
        title.setFont(new Font("Dialog", 0, 11));
        title.setForeground(Color.black);
        title.setHorizontalAlignment(SwingConstants.TRAILING);
        title.setBorder(BorderFactory.createEmptyBorder(3, 5, 3, 5));
        add(title, BorderLayout.SOUTH);

        updateLabel();
    }

    public Component getComponent()
    {
        return component;
    }

    public void setBounds(int x, int y, int width, int height)
    {
        // do not allow to change the location
        super.setBounds(INSETS, INSETS, width, height);

        // set the corresponding size of the editor
        editor.resetSize(width + 2 * INSETS, height + 2 * INSETS);

        // check and correct positions of the interfaces
        GrInterface[] ifaces = editor.getInterfaces(this);
        for(int i = 0; i < ifaces.length; i++)
            ifaces[i].reposition();

        // update layout info
        component.getLayout().setStandaloneSize(width, height);

        // fire layout evnet
        component.firePropertyChange("layout");
    }

    // VisualDesign implementation

    public boolean canMove()
    {
        return false;
    }

    public boolean canResize(int direction)
    {
        return direction == SwingConstants.SOUTH
                || direction == SwingConstants.SOUTH_EAST
                || direction == SwingConstants.EAST;
    }

    public Rectangle reviewMove(Rectangle before, Rectangle after)
    {
        return null;
    }

    public Rectangle reviewResize(Rectangle before, Rectangle after)
    {
        // respect interfaces
        GrInterface[] ifaces = editor.getInterfaces(this);
        for(int i = 0; i < ifaces.length; i++)
            after = ifaces[i].reviewFrameResize(after);
        // respect instances
        GrInstance[] instances = editor.getInstances();
        for(int i = 0; i < instances.length; i++)
            after = instances[i].reviewFrameResize(after);

        return after;
    }

    private ArchitectureEditor editor;
    private Component component;
    private JLabel title;

    private void updateLabel()
    {
        if(component.getType() != null)
            title.setText(component.getType().getName());
        else
            title.setText("null");
    }
}
