package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.actions;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.cookies.ComponentCookie;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.cookies.DataObjectCookie;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import org.openide.ErrorManager;
import org.openide.TopManager;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.actions.CookieAction;

import java.io.OutputStream;
import java.io.PrintWriter;

/**
 * This action generates the Component Description Language (CDL) file for
 * the project, describing the types build during the component development
 * process. <p>
 *
 * A CDL file is created and the repository is called to write the contained
 * types. Only types that are *not* commited are written.
 *
 * @author Michael Cifka
 */
public class ActGenerateCDL extends CookieAction
{
    public static final String CDL_EXT = "cdl";

    protected int mode()
    {
        return CookieAction.MODE_EXACTLY_ONE;
    }

    protected Class[] cookieClasses()
    {
        return new Class[]{DataObjectCookie.class};
    }

    public String getName()
    {
        return "Generate CDL File";
    }

    public HelpCtx getHelpCtx()
    {
        return HelpCtx.DEFAULT_HELP;
    }


    protected void performAction(Node[] nodes)
    {
        Node node = nodes[0];

        TopManager.getDefault().saveAll();

        DataObject data = ((DataObjectCookie) node.getCookie(DataObjectCookie.class)).getDataObject();
        String name = data.getPrimaryFile().getName();

        TopManager.getDefault().setStatusText("Generating " + name + ".cdl...");

        FileLock lock = null;
        try
        {
            FileObject directory = data.getPrimaryFile().getParent();
            FileObject file = directory.getFileObject(name, CDL_EXT);
            if(file == null)
            {
                // file does not exist; let's create it
                file = directory.createData(name, CDL_EXT);
            }
            lock = file.lock();
            OutputStream os = file.getOutputStream(lock);
            PrintWriter pw = new PrintWriter(os);
            Component component = ((ComponentCookie) node.getCookie(ComponentCookie.class)).getComponent();
            String s = component.getRepository().writeAsCDL();
            pw.print(s);
            pw.close();
            os.close();
            TopManager.getDefault().setStatusText(name + ".cdl was generated.");
        }
        catch (Exception e)
        {
            TopManager.getDefault().setStatusText("Error: " + name + ".cdl could not be generated.");
            ErrorManager.getDefault().notify(e);
        }
        finally
        {
            if(lock != null)
                lock.releaseLock();
        }
    }
}
