package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.nodes;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties.PropInterfaceArray;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties.PropInterfaceProtocol;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties.PropInterfaceSource;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties.PropInterfaceType;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Interface;
import org.openide.NotifyDescriptor;
import org.openide.nodes.PropertySupport.*;
import org.openide.TopManager;
import org.openide.nodes.*;

import java.beans.PropertyEditor;
import java.beans.PropertyEditorSupport;
import java.lang.reflect.InvocationTargetException;





/**
 * This node represents an interface. A large set of operations and
 * properties to manipulate the represented interface is provided.
 *
 * @author Michael Cifka
 */
public class InterfaceNode extends AbstractNode
{
    public InterfaceNode(Interface iface)
    {
        super(Children.LEAF);
        this.iface = iface;
    }

    protected Sheet createSheet()
    {
        Sheet s = Sheet.createDefault();
        Sheet.Set set = s.get(Sheet.PROPERTIES);
        set.put(new NameProp());
        set.put(createModeProperty());
        set.put(new PropInterfaceType(iface));
        set.put(new PropInterfaceSource(iface));
        set.put(new PropInterfaceProtocol(iface));
        set.put(new PropInterfaceArray(iface));
        return s;
    }

    private class NameProp extends PropertySupport.ReadWrite
    {
        public NameProp()
        {
            super("name", String.class, "Name", null);
        }

        public Object getValue()
        {
            return iface.getName();
        }

        public void setValue(Object val)
                throws IllegalArgumentException,
                InvocationTargetException
        {
            if(iface.getParent().getType().isCommited())
            {
                NotifyDescriptor d = new NotifyDescriptor.Message(
                        "Cannot modify a commited frame.",
                        NotifyDescriptor.INFORMATION_MESSAGE);
                TopManager.getDefault().notify(d);
                return;
            }
            iface.setName((String) val);
        }
    }

    private Node.Property createModeProperty()
    {
        Node.Property prop = new ReadWrite("mode", Integer.class, "Mode", null)
        {
            public Object getValue()
            {
                return new Integer(iface.getMode());
            }

            public void setValue(Object o)
            {
                if(iface.getParent().getType().isCommited())
                {
                    NotifyDescriptor d = new NotifyDescriptor.Message(
                            "Cannot modify a commited frame.",
                            NotifyDescriptor.INFORMATION_MESSAGE);
                    TopManager.getDefault().notify(d);
                    return;
                }
                iface.setMode(((Integer) o).intValue());
            }

            public PropertyEditor getPropertyEditor()
            {
                return new PropertyEditorSupport()
                {
                    public String[] getTags()
                    {
                        return new String[]{"provision", "requirement"};
                    }

                    public String getAsText()
                    {
                        if(iface.getMode() == Interface.PROVIDED)
                            return "provision";
                        else
                            return "requirement";
                    }

                    public void setAsText(String str)
                    {
                        if(iface.getParent().getType().isCommited())
                        {
                            NotifyDescriptor d = new NotifyDescriptor.Message(
                                    "Cannot modify a commited frame.",
                                    NotifyDescriptor.INFORMATION_MESSAGE);
                            TopManager.getDefault().notify(d);
                            return;
                        }

                        if(str.equals("provision"))
                            iface.setMode(Interface.PROVIDED);
                        else
                            iface.setMode(Interface.REQUIRED);
                    }
                };
            }
        };
        return prop;
    }

    private Interface iface;
}
