package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Architecture;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Component;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.typemanager.TypeEditor;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.ArchitectureType;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;


/**
 * A property representing the architecture type that was choosen
 * as the implementation for a component.
 *
 * @author Michael Cifka
 */
public class PropArchitecture extends PropertySupport.ReadWrite
{
    private Component comp;
    private TypeEditor editor;

    public PropArchitecture(Component c)
    {
        super("architecture", ArchitectureType.class, "Type", "Type of the architecture implementing this component.");
        setValue("changeImmediate", Boolean.FALSE);
        comp = c;
        editor = new TypeEditor(comp.getRepository());
        editor.addPropertyChangeListener(new PropertyChangeListener()
        {
            public void propertyChange(PropertyChangeEvent evt)
            {
                if(evt.getPropertyName() != null
                        && evt.getPropertyName().equals("setValue"))
                    setValue(editor.getValue());
            }
        });
        editor.setRequiredClass(ArchitectureType.class);
    }

    public Object getValue()
    {
        Architecture arch = comp.getImplementation();
        return (arch == null) ? null : arch.getType();
    }

    public void setValue(Object val)
    {
        if(val == null) return;
        if(comp.getImplementation() != null
                && comp.getImplementation().getType() == val)
            return;

        ArchitectureType type = (ArchitectureType) val;
        if(comp.getType() != type.getImplementedFrame())
        {
            if(type.getImplementedFrame() == null)
                type.setImplementedFrame(comp.getType());
            else
            {
                if(comp.getParent() != null && comp.getParent().getType().isCommited())
                {
                    NotifyDescriptor d = new NotifyDescriptor.Message(
                            "Cannot modify a commited frame.",
                            NotifyDescriptor.INFORMATION_MESSAGE);
                    TopManager.getDefault().notify(d);
                    return;
                }
                comp.setType(type.getImplementedFrame());
            }
        }
        Architecture impl = new Architecture(type, comp);
        comp.setImplementation(impl);
    }

    public PropertyEditor getPropertyEditor()
    {
        return editor;
    }
}