package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Interface;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyEditor;

/**
 * Using this property, the developer can provide a custom source code,
 * the protocol of the related interface. The code is entered via the provided
 * text editting window and it will be included in the generated CDL file,
 * under the protocol clause of the related interface.
 *
 * @author Michael Cifka
 */
public class PropInterfaceProtocol extends PropertySupport.ReadWrite
{
    private Interface iface;

    public PropInterfaceProtocol(Interface iface)
    {
        super("protocol", String.class, "Protocol", null);
        setValue("canEditAsText", Boolean.FALSE);
        setValue("changeImmediate", Boolean.FALSE);
        this.iface = iface;
    }

    public Object getValue()
    {
        return iface.getType().getProtocolCode();
    }

    public void setValue(Object val)
    {
        if(iface.getType().isCommited())
        {
            NotifyDescriptor d = new NotifyDescriptor.Message(
                    "Cannot modify a commited interface.",
                    NotifyDescriptor.INFORMATION_MESSAGE);
            TopManager.getDefault().notify(d);
            return;
        }
        iface.getType().setProtocolCode((String) val);
    }

    public PropertyEditor getPropertyEditor()
    {
        return new cz.cuni.dsrg.sofa.netbeans.visualdesigner.text.SourceCodeEditor();
    }
}