package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.Interface;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.typemanager.TypeEditor;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.ArchitectureType;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.InterfaceType;
import org.openide.NotifyDescriptor;
import org.openide.TopManager;
import org.openide.nodes.PropertySupport;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyEditor;

/**
 * A property representing the type that was choosen as the type of the interface.
 *
 * @author Michael Cifka
 */

public class PropInterfaceType extends PropertySupport.ReadWrite
{
    private Interface iface;

    private TypeEditor editor;

    public PropInterfaceType(Interface i)
    {
        super("interface", ArchitectureType.class, "Interface Type", "Type of this interface.");
        setValue("changeImmediate", Boolean.FALSE);
        iface = i;

        editor = new TypeEditor(i.getParent().getRepository());
        editor.addPropertyChangeListener(new PropertyChangeListener()
        {
            public void propertyChange(PropertyChangeEvent evt)
            {
                if(evt.getPropertyName() != null
                        && evt.getPropertyName().equals("setValue"))
                    setValue(editor.getValue());
            }
        });
        editor.setRequiredClass(InterfaceType.class);
    }

    public Object getValue()
    {
        return iface.getType();
    }

    public void setValue(Object val)
    {
        if(iface.getParent().getType().isCommited())
        {
            NotifyDescriptor d = new NotifyDescriptor.Message(
                    "Cannot modify a commited frame.",
                    NotifyDescriptor.INFORMATION_MESSAGE);
            TopManager.getDefault().notify(d);
            return;
        }
        iface.setType((InterfaceType) val);
    }

    public PropertyEditor getPropertyEditor()
    {
        return editor;
    }
}