package cz.cuni.dsrg.sofa.netbeans.visualdesigner.explorer.properties;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects.PropertiesProvider;

import javax.swing.*;

/**
 * This class is a JPanel, used to provide a custom property editor for
 * defining the properties of a component, variables predefined to customize
 * the behaviour of a component.
 *
 * Only properties of type Integer and String are implented.
 *
 * @author Michael Cifka
 */
public class PropertiesPanel extends javax.swing.JPanel
{

    private PropertiesProvider provider;


    // Creates new form PropertiesPanel
    public PropertiesPanel(PropertiesProvider provider)
    {
        this.provider = provider;
        initComponents();
        initList();

        nameField.getDocument().addDocumentListener(new javax.swing.event.DocumentListener()
        {
            public void changedUpdate(javax.swing.event.DocumentEvent e)
            {
                nameFieldTextChanged();
            }

            public void insertUpdate(javax.swing.event.DocumentEvent e)
            {
                nameFieldTextChanged();
            }

            public void removeUpdate(javax.swing.event.DocumentEvent e)
            {
                nameFieldTextChanged();
            }
        });
    }

    private void initList()
    {
        String[] properties = provider.getProperties();

        DefaultListModel model = new DefaultListModel();
        for(int i = 0; i < properties.length; i++)
        {
            model.add(i, properties[i] + " ("
                         + provider.getPropertyType(properties[i]) + ")");
        }
        propertiesList.setModel(model);
        propertiesListValueChanged(null);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents()
    {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        jPanel2 = new javax.swing.JPanel();
        propertiesList = new javax.swing.JList();
        jLabel1 = new javax.swing.JLabel();
        jPanel3 = new javax.swing.JPanel();
        nameField = new javax.swing.JTextField();
        typeBox = new javax.swing.JComboBox();
        jLabel2 = new javax.swing.JLabel();
        jLabel3 = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        addButton = new javax.swing.JButton();
        removeButton = new javax.swing.JButton();

        setLayout(new java.awt.GridBagLayout());

        jPanel2.setLayout(new java.awt.BorderLayout());

        jPanel2.setBorder(new javax.swing.border.EmptyBorder(new java.awt.Insets(20, 20, 20, 20)));
        propertiesList.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        propertiesList.setPreferredSize(new java.awt.Dimension(200, 200));
        propertiesList.addListSelectionListener(new javax.swing.event.ListSelectionListener()
        {
            public void valueChanged(javax.swing.event.ListSelectionEvent evt)
            {
                propertiesListValueChanged(evt);
            }
        });

        jPanel2.add(propertiesList, java.awt.BorderLayout.CENTER);

        jLabel1.setText("Properties:");
        jPanel2.add(jLabel1, java.awt.BorderLayout.NORTH);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.weighty = 1.0;
        add(jPanel2, gridBagConstraints);

        jPanel3.setLayout(new java.awt.GridBagLayout());

        nameField.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                nameFieldActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.weightx = 1.0;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 0);
        jPanel3.add(nameField, gridBagConstraints);

        typeBox.setModel(new javax.swing.DefaultComboBoxModel(new String[]{"Integer", "String"}));
        typeBox.setFont(new java.awt.Font("Dialog", 0, 12));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 20);
        jPanel3.add(typeBox, gridBagConstraints);

        jLabel2.setText("Name:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 20, 0, 0);
        jPanel3.add(jLabel2, gridBagConstraints);

        jLabel3.setText("Type:");
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.insets = new java.awt.Insets(20, 20, 0, 20);
        jPanel3.add(jLabel3, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        add(jPanel3, gridBagConstraints);

        jPanel4.setLayout(new java.awt.GridLayout(2, 0, 0, 5));

        addButton.setText("Add");
        addButton.setEnabled(false);
        addButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                addButtonActionPerformed(evt);
            }
        });

        jPanel4.add(addButton);

        removeButton.setText("Remove");
        removeButton.setEnabled(false);
        removeButton.addActionListener(new java.awt.event.ActionListener()
        {
            public void actionPerformed(java.awt.event.ActionEvent evt)
            {
                removeButtonActionPerformed(evt);
            }
        });

        jPanel4.add(removeButton);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        gridBagConstraints.insets = new java.awt.Insets(0, 20, 0, 20);
        add(jPanel4, gridBagConstraints);

    }//GEN-END:initComponents

    private void nameFieldActionPerformed(java.awt.event.ActionEvent evt)
    {//GEN-FIRST:event_nameFieldActionPerformed
        // Add your handling code here:
        if(addButton.isEnabled())
        {
            addButtonActionPerformed(null);
        }
    }//GEN-LAST:event_nameFieldActionPerformed

    private void propertiesListValueChanged(javax.swing.event.ListSelectionEvent evt)
    {//GEN-FIRST:event_propertiesListValueChanged
        // Add your handling code here:
        removeButton.setEnabled(propertiesList.getSelectedIndex() > -1);

    }//GEN-LAST:event_propertiesListValueChanged

    private void removeButtonActionPerformed(java.awt.event.ActionEvent evt)
    {//GEN-FIRST:event_removeButtonActionPerformed
        // Add your handling code here:
        Object[] sel = propertiesList.getSelectedValues();
        for(int i = 0; i < sel.length; i++)
        {
            String str = (String) sel[i];
            String name = str.substring(0, str.lastIndexOf(" ("));
            provider.removeProperty(name);
        }
        initList();
    }//GEN-LAST:event_removeButtonActionPerformed

    private void addButtonActionPerformed(java.awt.event.ActionEvent evt)
    {//GEN-FIRST:event_addButtonActionPerformed
        // Add your handling code here:
        String[] types = new String[]{"integer", "string"};
        provider.addProperty(nameField.getText(), types[typeBox.getSelectedIndex()]);
        initList();
        nameField.setText("");
    }//GEN-LAST:event_addButtonActionPerformed


    private void nameFieldTextChanged()
    {
        addButton.setEnabled(!nameField.getText().equals(""));
    }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JComboBox typeBox;
    private javax.swing.JTextField nameField;
    private javax.swing.JButton addButton;
    private javax.swing.JLabel jLabel3;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JList propertiesList;
    private javax.swing.JButton removeButton;
    // End of variables declaration//GEN-END:variables

}
