package cz.cuni.dsrg.sofa.netbeans.visualdesigner.objects;

/**
 * This class represents a binding between two interface used in
 * a software component.<p>
 *
 * Methods to set the operands, and the parent (i.e. the holding
 * architecture) are provided.<p>
 *
 * This class can be persisted using the Java Object Serialization.
 *
 * @author Michael Cifka
 */
public class Binding implements java.io.Serializable
{

    public static final int BIND = 0;
    public static final int SUBSUME = 1;
    public static final int DELEGATE = 2;

    public Binding(int mode, Interface lhsOperand, Interface rhsOperand, Architecture parent)
    {
        this.mode = mode;
        this.lhsOperand = lhsOperand;
        this.rhsOperand = rhsOperand;
        this.parent = parent;
        lhsOperand.addBinding(this);
        rhsOperand.addBinding(this);
    }

    public Binding(Interface lhsOperand, Interface rhsOperand, Architecture parent)
    {
        // suppose that lhsOperand.getParent() != rhsOperand.getParent()
        this(
                (lhsOperand.getParent() == parent.getParent()) ? DELEGATE :
                (rhsOperand.getParent() == parent.getParent()) ? SUBSUME : BIND,
                lhsOperand, rhsOperand, parent);
    }

    public int getMode()
    {
        return mode;
    }

    public Interface getLhsOperand()
    {
        return lhsOperand;
    }

    public Interface getRhsOperand()
    {
        return rhsOperand;
    }

    public Architecture getParent()
    {
        return parent;
    }

    private int mode;
    private Interface lhsOperand;
    private Interface rhsOperand;
    private Architecture parent;
}