package cz.cuni.dsrg.sofa.netbeans.visualdesigner.typemanager;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.AModule;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.AbstractType;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.Module;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.Repository;

import javax.swing.*;
import java.awt.datatransfer.Clipboard;
import java.awt.event.ActionEvent;

/**
 * This is a helper class that creates the toolbar for the type manager.
 *
 * @author Michael Cifka
 */
public class Actions
{

    public Actions(TypeManager manager)
    {
        this.manager = manager;
    }

    private TypeManager manager;

    private Action synchronizeAction = new AbstractAction(
            "Synchronize", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/synchronize.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.synchronize();
        }
    };
    private Action folderAction = new AbstractAction(
            "New Module", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/folderNew.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.newFolder();
        }
    };
    private Action interfaceAction = new AbstractAction(
            "New Interface", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/interfaceNew.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.newInterface();
        }
    };
    private Action frameAction = new AbstractAction(
            "New Frame", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/frameNew.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.newFrame();
        }
    };
    private Action providerAction = new AbstractAction(
            "New Provider", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/providerNew.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.newProvider();
        }
    };
    private Action architectureAction = new AbstractAction(
            "New Architecture", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/architectureNew.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.newArchitecture();
        }
    };
    private Action duplicateAction = new AbstractAction(
            "Duplicate", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/duplicate.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.duplicate();
        }
    };
    private Action cutAction = new AbstractAction(
            "Cut", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/cut.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.cut();
        }
    };
    private Action copyAction = new AbstractAction(
            "Copy", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/copy.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.copy();
        }
    };
    private Action pasteAction = new AbstractAction(
            "Paste", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/paste.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.paste();
        }
    };
    private Action deleteAction = new AbstractAction(
            "Delete", new ImageIcon(getClass().getResource(
                    "/cz/cuni/dsrg/sofa/netbeans/resources/visualdesigner/icons/delete.gif")))
    {
        public void actionPerformed(ActionEvent e)
        {
            manager.delete();
        }
    };

    public JToolBar createToolBar()
    {
        JToolBar toolBar = new JToolBar(JToolBar.HORIZONTAL);
        toolBar.add(synchronizeAction);
        toolBar.addSeparator();
        toolBar.add(folderAction);
        toolBar.add(interfaceAction);
        toolBar.add(frameAction);
        toolBar.addSeparator();
        toolBar.add(providerAction);
        toolBar.add(architectureAction);
        toolBar.addSeparator();
        toolBar.add(duplicateAction);
        toolBar.add(cutAction);
        toolBar.add(copyAction);
        toolBar.add(pasteAction);
        toolBar.add(deleteAction);
        toolBar.setFloatable(false);
        toolBar.setBorder(BorderFactory.createEmptyBorder(5, 0, 5, 5));
        return toolBar;
    }

    public void enable()
    {

        TypeNode node = manager.getSelectedNode();
        AbstractType type = manager.getSelectedType();

        if(node != null && manager != null)
        {
            folderAction.setEnabled(type.isContainer());
            interfaceAction.setEnabled(type instanceof Module);
            frameAction.setEnabled(type instanceof Module);
            providerAction.setEnabled(type instanceof Repository);
            architectureAction.setEnabled(type instanceof AModule);
            duplicateAction.setEnabled(!type.isContainer());

            cutAction.setEnabled(node.canCut());
            copyAction.setEnabled(node.canCopy());
            deleteAction.setEnabled(node.canDestroy());

            boolean pasteEnabled = false;
            if(type.isContainer())
            {
                java.awt.datatransfer.Clipboard clipboard = getClipboard();
                java.awt.datatransfer.Transferable trans = clipboard.getContents(this);
                if(trans != null)
                    pasteEnabled = trans.isDataFlavorSupported(TypeNode.TYPE_FLAVOR);
            }
            pasteAction.setEnabled(pasteEnabled);
        }
        else
        {
            folderAction.setEnabled(false);
            interfaceAction.setEnabled(false);
            frameAction.setEnabled(false);
            providerAction.setEnabled(false);
            architectureAction.setEnabled(false);
            duplicateAction.setEnabled(false);
            cutAction.setEnabled(false);
            copyAction.setEnabled(false);
            deleteAction.setEnabled(false);
        }
    }

    private static Clipboard getClipboard()
    {
        Clipboard c = (java.awt.datatransfer.Clipboard)
                org.openide.util.Lookup.getDefault().lookup(java.awt.datatransfer.Clipboard.class);
        if(c == null)
        {
            c = java.awt.Toolkit.getDefaultToolkit().getSystemClipboard();
        }
        return c;
    }
}