package cz.cuni.dsrg.sofa.netbeans.visualdesigner.typemanager;

import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.AbstractType;
import cz.cuni.dsrg.sofa.netbeans.visualdesigner.types.Repository;
import org.openide.explorer.propertysheet.ExPropertyEditor;
import org.openide.explorer.propertysheet.PropertyEnv;
import org.openide.util.WeakListener;

import java.awt.*;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

/**
 * This class can be used with properties to select a desired type,
 * using the type manager. A required class of type (frame, architecture,
 * interface, etc.) can be set.
 *
 * @author Michael Cifka
 */
public class TypeEditor implements ExPropertyEditor, PropertyChangeListener
{

    private PropertyChangeSupport supp = new PropertyChangeSupport(this);
    private TypeManager typeManager;
    private AbstractType value;
    private Class requiredClass = null;

    public TypeEditor(Repository repository)
    {
        typeManager = TypeManager.get(repository);
        typeManager.addPropertyChangeListener(
                WeakListener.propertyChange(this, typeManager));
    }

    public void setRequiredClass(Class requiredClass)
    {
        this.requiredClass = requiredClass;
    }

    public void addPropertyChangeListener(PropertyChangeListener l)
    {
        supp.addPropertyChangeListener(l);
    }

    public void removePropertyChangeListener(PropertyChangeListener l)
    {
        supp.removePropertyChangeListener(l);
    }

    public boolean supportsCustomEditor()
    {
        return true;
    }

    public Component getCustomEditor()
    {
        typeManager.setSelectedType(value);
        return typeManager;
    }

    public void propertyChange(PropertyChangeEvent ev)
    {
        if(ev.getPropertyName().equals(TypeManager.PROP_SELECTED_TYPE))
        {
            AbstractType t = typeManager.getSelectedType();
            if(requiredClass == null || requiredClass.isInstance(t))
            {
                value = t;
                supp.firePropertyChange(ExPropertyEditor.PROP_VALUE_VALID, null, Boolean.TRUE);
            }
            else
            {
                supp.firePropertyChange(ExPropertyEditor.PROP_VALUE_VALID, null, Boolean.FALSE);
            }
            supp.firePropertyChange(null, null, null);
        }
    }

    public Object getValue()
    {
        return value;
    }

    public void setValue(Object o)
    {
        if(o != null || o instanceof AbstractType)
        {
            value = (AbstractType) o;
        }
        else
            value = null;
    }

    public String getAsText()
    {
        AbstractType type = (AbstractType) getValue();
        if(type == null)
        {
            return "(not specified)";
        }
        else
        {
            return type.getName();
        }
    }

    public void setAsText(String text)
    {
        AbstractType type = (AbstractType) getValue();
        if(type == null || text.equals("") || text.equals(type.getName()))
            return;

        AbstractType collision = type.getParent().getDefaultChild(text);
        if(collision != null && (!collision.isCommited() || type.isContainer()
                || !type.getClass().isInstance(collision)))
            return;

        type.setName(text);
        typeManager.refresh();
        supp.firePropertyChange("setValue", null, null);
    }


    public String getJavaInitializationString()
    {
        return null;
    }

    public String[] getTags()
    {
        return null;
    }

    public boolean isPaintable()
    {
        return false;
    }

    public void paintValue(Graphics g, Rectangle r)
    {
    }

    public void attachEnv(PropertyEnv propertyEnv)
    {
    }
}