package cz.cuni.dsrg.sofa.netbeans.visualdesigner.types;

import SOFA.SOFAnode.Made.TIR.*;

import java.rmi.RemoteException;

/**
 * This class extends the {@link AbstractType} to hold, import, and export
 * the type information representing an array of a type.<p>
 *
 * Reference to the type of the array elements is managed.<p>
 *
 * This class can be persisted using the Java Object Serialization.
 *
 * @author Michael Cifka
 */
public class ArrayType extends AbstractType
{
    ArrayType()
    {
        container = false;
    };

    ArrayType(AbstractType element, String property)
    {
        this();
        this.element = element;
        this.property = property;
        String base = element.getName() + "Array";
        int counter = 1;
        name = base;
        while(element.getParent().getDefaultChild(name) != null)
            name = base + counter++;
    }

    public AbstractType getElement()
    {
        return element;
    }

    public String getProperty()
    {
        return property;
    }

    protected void loadCommitedObjects(TIRObject o)
    {
        try
        {
            CDLType type = ((TypedefDef) o).original_type();
            if(!(type instanceof ArrayDef))
                throw new NotSupportedException();

            ArrayDef arr = (ArrayDef) type;
            ExprOperDef size = arr.length();
            if(!(size instanceof ExprProperty))
                throw new NotSupportedException();

            property = ((ExprProperty) size).ref_prop();
            CDLType elementType = arr.element_type();
            if(elementType instanceof InterfaceDef
                    || elementType instanceof FrameDef)
            {
                String elementFullName = getFullName((Contained) elementType);
                element = root.findObject(elementFullName);
                if(element instanceof InterfaceType)
                    ((InterfaceType) element).putArrayType(this);
                else
                    ((FrameType) element).putArrayType(this);
            }
            else
                throw new NotSupportedException();
        }
        catch (Exception e)
        {
            System.out.println("Exception: " + e.getMessage());
            e.printStackTrace();
            if(e instanceof NotSupportedException)
                throw (NotSupportedException) e;
        }
    }

    String writeAsCDL(int indent)
    {
        String indentString = indentString(indent);
        String s = indentString + "typedef " + element.getName()
                + " " + name + "[$" + property + "];\n";
        return s;
    }

    private String getFullName(Contained obj) throws RemoteException
    {
        Identification id = obj.get_identification();
        String absoluteName = id.absolute_name().name();
        return absoluteName.substring(absoluteName.indexOf("::"))
                + " ver. " + id.version();
    }

    private String property;
    private AbstractType element;
}