package cz.cuni.dsrg.sofa.netbeans.visualdesigner.types;

import java.util.Map;
import java.util.TreeMap;

/**
 * This class extends the {@link AbstractType} to hold, import, and export
 * the type information representing an interface type.<p>
 *
 * This class can be persisted using the Java Object Serialization.
 *
 * @author Michael Cifka
 */
public class InterfaceType extends AbstractType implements java.io.Serializable
{

    static final long serialVersionUID = -5571969844921472668L;

    private Map arrayTypes;

    private String sourceCode;
    private String protocolCode;

    InterfaceType()
    {
        container = false;
        arrayTypes = new TreeMap();
    };

    public InterfaceType(String name)
    {
        this();
        this.name = name;
    }

    public ArrayType getArrayType(String property)
    {
        ArrayType t = (ArrayType) arrayTypes.get(property);
        if(t == null)
        {
            t = new ArrayType(this, property);
            parent.addChild(t);
            arrayTypes.put(property, t);
        }
        return t;
    }

    public AbstractType duplicate(AbstractType parent)
    {
        return new InterfaceType(parent.getAvailableName(name, InterfaceType.class));
    }

    public void setSourceCode(String s)
    {
        sourceCode = s;
    }

    public String getSourceCode()
    {
        return sourceCode;
    }

    public void setProtocolCode(String s)
    {
        protocolCode = s;
    }

    public String getProtocolCode()
    {
        return protocolCode;
    }

    void putArrayType(ArrayType arr)
    {
        arrayTypes.put(arr.getProperty(), arr);
    }


    String writeAsCDL(int indent)
    {

        if(commited) return "";

        String indentString = indentString(indent);
        String s = "\n" + indentString + "interface " + getName() + " {\n";
        String t = "";
        if(sourceCode != null && !sourceCode.equals(""))
            t += indentCode(sourceCode, indent + 1);
        if(protocolCode != null && !protocolCode.equals(""))
        {
            t += "\n" + indentString(indent + 1) + "protocol:\n";
            t += indentCode(protocolCode, indent + 2);
        }

        s += t.equals("") ? "\n" : t;
        s += indentString + "};\n";

        ArrayType[] arr = (ArrayType[]) arrayTypes.values().toArray(new ArrayType[0]);
        for(int i = 0; i < arr.length; i++)
            s += "\n" + arr[i].writeAsCDL(indent);

        return s;
    }
}