package cz.cuni.dsrg.sofa.netbeans.visualdesigner.types;

import SOFA.SOFAnode.Made.TIR.Container;
import cz.cuni.dsrg.sofa.netbeans.settings.VDSettings;
import cz.cuni.dsrg.sofa.netbeans.tirbrowser.MiscHelps;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.rmi.Naming;
import java.util.HashMap;
import java.util.Map;

/**
 * This class extends the {@link AbstractType} is representing the top-level
 * folder of the type-information.<p>
 *
 * It implements the functionality to connect to the type repository and
 * read the contained information, as well as to create new types. For the
 * purpose of creating new types automatically (e.g. when performing the visual
 * editation in the component designer), references to default locations
 * where the new types should be places are kept.<p>
 *
 * This class can be persisted using the Java Object Serialization.
 *
 * @author Michael Cifka
 */
public class Repository extends Module implements java.io.Serializable, PropertyChangeListener
{

 public static VDSettings settings = (VDSettings) org.openide.options.SystemOption.findObject(VDSettings.class, true);

    public static void main(String[] args)
    {
    }

    public Repository()
    {
        name = "Repository";
        commited = true;
        root = this;
        index = new HashMap();
        settings.addPropertyChangeListener(this);
        getDefaultModule();
        getDefaultProvider();
    }

    public String getScope()
    {
        return "";
    }

    public String getScopeName()
    {
        return "";
    }

    public String getScopeNameVersion()
    {
        return "::";
    }

    public void synchronize()
    {
        loadCommitedObjects();
    }

    public void clean()
    {
        index = new HashMap();
        removeAllChildren();
        defaultModule = null;
        defaultProvider = null;
        setDefaultModule(null);
        setDefaultProvider(null);
    }

    public String getFullName()
    {
        // return "::";
        return "";
    }

    public AbstractType findObject(String fullName)
    {
        return (AbstractType) index.get(fullName);
    }

    public InterfaceType createInterfaceType()
    {
        AbstractType module = getDefaultModule();
        String s;
        for(int i = 1; true; i++)
        {
            s = "Interface" + i;
            if(module.getDefaultChild(s) == null) break;
        }
        InterfaceType iface = new InterfaceType(s);
        module.addChild(iface);
        return iface;
    }

    public FrameType createFrameType()
    {
        AbstractType module = getDefaultModule();
        String s;
        for(int i = 1; true; i++)
        {
            s = "Frame" + i;
            if(module.getDefaultChild(s) == null) break;
        }
        FrameType frame = new FrameType(s);
        module.addChild(frame);
        return frame;
    }

    public ArchitectureType createArchitectureType(FrameType implementedFrame)
    {
        AbstractType aModule = getDefaultProvider();
        String s;
        for(int i = 1; true; i++)
        {
            s = "Architecture" + i;
            if(aModule.getDefaultChild(s) == null) break;
        }
        ArchitectureType arch = new ArchitectureType(s, implementedFrame);
        aModule.addChild(arch);
        return arch;
    }

    private Module defaultModule = null;

    public Module getDefaultModule()
    {
       if(defaultModule==null)
        setDefaultModule(null);

       return defaultModule;
    }

    public void setDefaultModule(String module)
    {
       String[] mods;

        // refrerences are used to prevent the module from being removed

        if(defaultModule!=null) defaultProvider.removeReference();

         if(module==null) module= settings.getModule();
         else if((defaultModule!=null) && module.equalsIgnoreCase(defaultModule.getFullName()))
                return;

        mods= module.split("::");
        if(mods.length==0)
         if(module.equalsIgnoreCase(""))
        {
            defaultModule= new Module("");
            defaultModule.root = this;
            defaultModule.parent= this;
            this.addChild(defaultModule);
         }
         else
         { defaultModule= new Module(module);
           defaultModule.root= this;
           defaultModule.parent= this;
           this.addChild(defaultModule);
         }
        else
         { defaultModule = new Module(mods[0]);
            defaultModule.root= this;
            defaultModule.parent= this;
            this.addChild(defaultModule);
           Module tmpModule;
           for(int i=1; i<mods.length; i++)
           { tmpModule= new Module(mods[i]);
             defaultModule.addChild(tmpModule);
             defaultModule= tmpModule;
           }
         }
         defaultModule.addReference();
    }

    private AModule defaultProvider = null;

    public AModule getDefaultProvider()
    {
        if(defaultProvider==null)
         setDefaultProvider(null);

        return defaultProvider;
    }

    public void setDefaultProvider(String aModule)
    {
        String[] mods;

       // refrerences are used to prevent the aModule from being removed
        if(defaultProvider!=null) defaultProvider.removeReference();

        if(aModule==null) aModule= settings.getProvider();
        else if((defaultProvider!=null) && aModule.equalsIgnoreCase(defaultProvider.getFullName()))
               return;

        mods= aModule.split("::");

        if(mods.length==0)
         if(aModule.equalsIgnoreCase(""))
         { defaultProvider= new Provider("CUNI");
           defaultProvider.root= this;
           this.addChild(defaultProvider);
         }
         else
         { defaultProvider= new Provider(aModule);
           defaultProvider.root= this;
           defaultProvider.parent= this;
           this.addChild(defaultProvider);
         }
        else
         { defaultProvider = new Provider(mods[0]);
           defaultProvider.root= this;
           defaultProvider.parent= this;
           this.addChild(defaultProvider);
           AModule tmpModule;
           for(int i=1; i<mods.length; i++)
           { tmpModule= new AModule(mods[i]);
             defaultProvider.addChild(tmpModule);
             defaultProvider= tmpModule;
           }
         }

        defaultProvider.addReference();
    }

    private String customCode;

    public void setCustomCode(String s)
    {
        customCode = s;
    }

    public String getCustomCode()
    {
        return customCode;
    }

    void indexObject(String fullName, AbstractType obj)
    {
        index.put(fullName, obj);
    }

    void removeIndex(String fullName)
    {
        index.remove(fullName);
    }

    private void loadCommitedObjects()
    {
        String rmiport = System.getProperty("tir.rmiport", "2000");
        String rmihost = System.getProperty("tir.rmihost", "localhost");
        SOFA.SOFAnode.Made.TIR.Repository rep;
        try
        {
            rep = (SOFA.SOFAnode.Made.TIR.Repository) Naming.lookup(
                    "//" + rmihost + ":" + rmiport + "/Repository");
        }
        catch (Exception e)
        {
            throw new RepositoryException("Cannot connect to the repository.");
        }
        try
        {
            loadCommitedObjects((Container) rep.contents(null)[0]);
        }
        catch (Exception e)
        {
            throw new RepositoryException("Error in communication with the repository.");
        }
    }

    public String writeAsCDL()
    {

        String s = "";
        if(customCode != null && !customCode.equals(""))
            s += customCode + "\n\n";

        s += writeInterfaceTypesAsCDL();

       String t = writeFrameTypesAsCDL();

MiscHelps.message("Repository  FRAMES WRITTEN");

        s += (s.equals("")) ? t : "\n" + t;

        // write providers
        AbstractType[] ch = getChildren();
        for(int i = 0; i < ch.length; i++)
        {

            if(ch[i] instanceof Provider)
            {
                s += (s.equals("")) ? "" : "\n";
                s += ch[i].writeAsCDL();
            }
        }
        return s;
    }

    public String writeInterfaceTypesAsCDL()
    {
        String s = "";
        AbstractType[] ch = getChildren();
//*MiscHelps.message("WriteInterfaceTypes: " + Integer.toString(ch.length));
        // standalone interface types
        for(int i = 0; i < ch.length; i++)
        {
//*MiscHelps.message("WriteInterfaceTypes: " + ch[i].getName());
            if(ch[i] instanceof InterfaceType)
                s += ch[i].writeAsCDL(0) + "\n";
        }
        // interface types in modules
        for(int i = 0; i < ch.length; i++)
        {
            if(ch[i] instanceof Module)
            {
                s += ((Module) ch[i]).writeInterfaceTypesAsCDL(0);
            }
        }
        return s;
    }

    public String writeFrameTypesAsCDL()
    {
        String s = "";
        AbstractType[] ch = getChildren();
//*MiscHelps.message("FrameTypes: " + Integer.toString(ch.length));
        // standalone frame types
        for(int i = 0; i < ch.length; i++)
        {
//*MiscHelps.message("WriteFrameTypes: " +ch[i].getName());
            if((ch[i] instanceof FrameType) && !ch[i].getName().equalsIgnoreCase("SYSTEM"))
             s += ch[i].writeAsCDL(0) + "\n";
         }
        // frame types in modules
        for(int i = 0; i < ch.length; i++)
        {
         if(ch[i] instanceof Module)
                s += ((Module) ch[i]).writeFrameTypesAsCDL(0);

        }
        return s;
    }




    private Map index;

    public void propertyChange(PropertyChangeEvent evt)
    {


      if(evt.getPropertyName()==VDSettings.PROP_PROVIDER)
     {
        setDefaultProvider(null);
     }

      if(evt.getPropertyName()==VDSettings.PROP_MODULE)
      {

        setDefaultModule(null);
      }

/*         AbstractType[] deti1= defaultModule.getChildren();
         AbstractType[] deti2= defaultProvider.getChildren();


        removeChild(defaultModule);
        removeChild(defaultProvider);

         defaultModule= null;
         defaultProvider=null;
         setDefaultModule(null);
         setDefaultProvider(null);

        for(int i=0; i<deti1.length; i++)
           deti1[i].parent= defaultModule;

        for(int i=0; i<deti2.length; i++)
        deti2[i].parent= defaultProvider;

     }
     else
          this.removeChild(defaultModule);
          defaultModule= null;
          setDefaultModule(null);
          for(int i=0; i<deti.length; i++)
          deti[i].parent= defaultModule;
      }

 */
 }
}

